// IMKit-Anthy: A Qtopia InputMethod interface for Anthy
// Copyright (C) 2002  YamaKen <yamaken@bp.iij4u.or.jp>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

// $Name: IMKIT_ANTHY_0_3_5 $
// $Id: ui_std.cpp,v 1.7 2002/11/26 17:47:38 yamaken Exp $

#include <qfont.h>
#include "ui_std.h"
#include "imwidget.h"
#include "widgets/imwidget_std.h"
#include "widgets/imkit_qlineedit.h"

void
StdIMWidgetFactory::initialize_imwidget(QWidget *imwidget) {
  imwidget = imwidget;  //to supress warning
}

void
StdIMWidgetFactory::initialize_preedit(Preedit *preedit) {
  preedit = preedit;  //to supress warning
}

void
StdIMWidgetFactory::initialize_inputmap_indicator(InputMapIndicator *indicator)
{
  indicator->add_state(IMKIT_INPUT_MAP_HIRAGANA,
                       StdInputMapIndicator::tr("hira"));
  indicator->add_state(IMKIT_INPUT_MAP_KATAKANA,
                       StdInputMapIndicator::tr("kata"));
  indicator->add_state(IMKIT_INPUT_MAP_ALPHA,
                       StdInputMapIndicator::tr("alnm"));
  indicator->add_state(IMKIT_INPUT_MAP_WALPHA,
                       StdInputMapIndicator::tr("walnm"));
}

void
StdIMWidgetFactory::initialize_candidate_window(CandidateWindow *cand_win) {
  cand_win = cand_win;  //to supress warning
}

PreeditWidget *
StdIMWidgetFactory::create_preedit_widget(QWidget *parent,
                                          Qt::WFlags flags,
                                          const char *name)
{
  IMKitQLineEdit *widget;
  PreeditWidget *abstract_widget;

  widget = new IMKitQLineEdit(parent, name);
  abstract_widget = new PreeditWidgetAbstractor<IMKitQLineEdit>(widget);

  return abstract_widget;
}

InputMapIndicatorWidget *
StdIMWidgetFactory::create_inputmap_indicator_widget(QWidget *parent,
                                                     Qt::WFlags flags,
                                                     const char *name)
{
  QComboBox *widget;
  InputMapIndicatorWidget *abstract_widget;

  widget = new QComboBox(parent, name);
  abstract_widget = new InputMapIndicatorWidgetAbstractor<QComboBox>(widget);
  // TODO: indicator_widget->adjustSize();  //add_state()˼¹

  return abstract_widget;
}

CandidateWindowWidget *
StdIMWidgetFactory::create_candidate_window_widget(QWidget *parent,
                                                   Qt::WFlags flags,
                                                   const char *name)
{
  QListBox *widget;
  CandidateWindowWidget *abstract_widget;
  Qt::WFlags overwrite_flags;

  overwrite_flags = (Qt::WType_TopLevel
                     | Qt::WStyle_Customize
                     | Qt::WStyle_NoBorderEx
                     | Qt::WStyle_StaysOnTop
                     );
  widget = new QListBox(parent, name, overwrite_flags);
  widget->setVScrollBarMode(QListBox::AlwaysOn);
  widget->setSelectionMode(QListBox::Single);
  widget->setColumnMode(1);
#ifdef IMKIT_FIXED_SIZE_CANDIDATE_WINDOW
  widget->setResizePolicy(QListBox::Manual);
  widget->resize(160, 150);
#else
  //widget->setResizePolicy(QListBox::AutoOneFit);
#endif
  imkit_set_font(widget);

  abstract_widget = new CandidateWindowWidgetAbstractor<QListBox>(widget);

  return abstract_widget;
}

StdIMWidgetFactory::StdIMWidgetFactory(void) {
}

QWidget *
StdIMWidgetFactory::create_imwidget(QWidget *parent,
                                    Qt::WFlags flags,
                                    const char *name)
{
  QWidget *imwidget;

  imwidget = new StdIMWidget(parent, name, flags);
  initialize_imwidget(imwidget);

  return imwidget;
}

Preedit *
StdIMWidgetFactory::create_preedit(QWidget *parent,
                                   Qt::WFlags flags,
                                   const char *name)
{
  PreeditWidget *abstract_widget;
  Preedit *preedit;

  abstract_widget = create_preedit_widget(parent, flags, name);
  preedit = new StdPreedit(abstract_widget, QChar(IMKIT_NON_UNICODE_CHAR));
  initialize_preedit(preedit);

  return preedit;
}

InputMapIndicator *
StdIMWidgetFactory::create_inputmap_indicator(QWidget *parent,
                                              Qt::WFlags flags,
                                              const char *name)
{
  InputMapIndicatorWidget *abstract_widget;
  StdInputMapIndicator *indicator;

  abstract_widget = create_inputmap_indicator_widget(parent, flags, name);
  indicator = new StdInputMapIndicator(abstract_widget);
  initialize_inputmap_indicator(indicator);

  return indicator;
}

CandidateWindow *
StdIMWidgetFactory::create_candidate_window(QWidget *parent,
                                            Qt::WFlags flags,
                                            const char *name)
{
  CandidateWindowWidget *abstract_widget;
  CandidateWindow *cand_win;

  abstract_widget = create_candidate_window_widget(parent, flags, name);
  cand_win = new StdCandidateWindow(abstract_widget);
  initialize_candidate_window(cand_win);

  return cand_win;
}

void
StdIMUserInterface::delete_keyfilter(void) {
  CascadeKeyFilter *i, *next;
  
  for (i = keyfilter_begin()->next(); i != keyfilter_end(); i = next) {
    next = i->next();
    i->remove();
    delete i;
  }
}

void
StdIMUserInterface::connect_signals(void) {
  connect(_engine, SIGNAL(preedit_state_changed(PreeditState)),
          this, SLOT(update_preedit_state(PreeditState)));

  connect(_engine, SIGNAL(committed(const QString &)),
          injector, SLOT(inject_str(const QString &)));
  connect(_engine, SIGNAL(map_changed(InputMap)),
          indicator, SLOT(select(InputMap)));
  connect(_engine, SIGNAL(map_state_added(InputMap, const QString &)),
          indicator, SLOT(add_state(InputMap, const QString &)));
  connect(_engine, SIGNAL(map_states_cleared(void)),
          indicator, SLOT(clear(void)));
  
  connect(_engine, SIGNAL(preedit_state_changed(PreeditState)),
          preedit, SLOT(update_state(PreeditState)));
  connect(_engine, SIGNAL(segments_changed(Segments &)),
          preedit, SLOT(update(Segments &)));

  connect(_engine, SIGNAL(update_candidates(Candidates &)),
          cand_win, SLOT(update(Candidates &)));
  connect(_engine, SIGNAL(hilight_candidate(int, bool)),
          cand_win, SLOT(hilight(int, bool)));
  connect(_engine, SIGNAL(activation_hint_for_candidates(bool)),
          cand_win, SLOT(slot_activate(bool)));

  connect(indicator, SIGNAL(selected(InputMap)),
          _engine, SLOT(select_map(InputMap)));
  connect(cand_win, SIGNAL(selected(Candidates::size_type)),
          _engine, SLOT(select_candidate(Candidates::size_type)));
  connect(preedit, SIGNAL(focus_changed(const QRect &)),
          cand_win, SLOT(move_adjacent(const QRect &)));

  connect(_imwidget, SIGNAL(activated(bool)),
          this, SLOT(slot_activate(bool)));
}

TranslationEngine &
StdIMUserInterface::engine(void) {
  return *_engine;
}

CommandMap *
StdIMUserInterface::ui_command_map(void) {
  CommandMap *_ui_command_map;

  _ui_command_map = new CommandMap;
  (*_ui_command_map)["do_nothing"] = new DoNothingCommand("do_nothing");
  //"activate_candidate_window"
  //"deactivate_candidate_window"
  //"purge_engine"

  return _ui_command_map;
}

StdIMUserInterface::
StdIMUserInterface(TranslationEngine *engine_init,
                   StdIMWidgetFactory *imwidget_factory_init)
  : _engine(engine_init), _command_map(NULL),
    _keyfilter_begin(NULL), _keyfilter_end(NULL), injector(NULL),
    preedit(NULL), indicator(NULL), cand_win(NULL), _imwidget(NULL),
    imwidget_factory(imwidget_factory_init)
{
  _keyfilter_begin = new DumbPipe;
  _keyfilter_end = new DumbPipe;
  _keyfilter_begin->insert_after(_keyfilter_end);
  //assemble_engine();
}

StdIMUserInterface::~StdIMUserInterface(void) {
  purge_engine();
  delete _keyfilter_begin;
  delete _keyfilter_end;
  delete imwidget_factory;
}

CascadeKeyFilter *
StdIMUserInterface::keyfilter_begin(void) {
  return _keyfilter_begin;
}

CascadeKeyFilter *
StdIMUserInterface::keyfilter_end(void) {
  return _keyfilter_end;
}

QWidget *
StdIMUserInterface::imwidget(void) {
  return _imwidget;
}

QWidget *
StdIMUserInterface::imwidget(QWidget *parent, Qt::WFlags flags) {
  if (!_imwidget) {
    _imwidget = imwidget_factory->create_imwidget(parent, flags);
    preedit = imwidget_factory->create_preedit(_imwidget, flags);
    indicator = imwidget_factory->create_inputmap_indicator(_imwidget, flags);
    cand_win = imwidget_factory->create_candidate_window(preedit->widget(),
                                                         flags);
    //connect_signals();
    assemble_engine();
  }

  return _imwidget;
}

CommandMap *
StdIMUserInterface::command_map(void) {
  CommandMap *engine_commands, *ui_commands;

  if (!_command_map) {
    _command_map = new CommandMap;

    engine_commands = engine().command_map();
    ui_commands = ui_command_map();

    _command_map->insert(engine_commands->begin(), engine_commands->end());
    _command_map->insert(ui_commands->begin(), ui_commands->end());
  }

  return _command_map;
}

void
StdIMUserInterface::assemble_engine(void) {
  _keyfilter_begin->insert_after(create_keyfilter());
  //_engine = new AnthyEngine;  //TODO: ưŪengine
  connect_signals();
  deactivate();
}

void
StdIMUserInterface::purge_engine(void) {
#if 0
  //~StdIMUserInterfaceƤФȡλset_activity()
  //̵ˤʤäƤ
  deactivate();
#endif
  disconnect(preedit, SIGNAL(focus_changed(const QRect &)),
             cand_win, SLOT(move_adjacent(const QRect &)));
  delete_keyfilter();

  {
    CommandMap *engine_commands;
    CommandMap::iterator i;
    
    engine_commands = engine().command_map();
    for (i = engine_commands->begin(); i != engine_commands->end(); i++) {
      command_map()->erase(i->first);
    }
  }
  imkit_delete_command_map(_command_map);
  _command_map = NULL;

  delete _engine;
  _engine = NULL;
}
